/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - convert.c
 *	---------------------------------
 *
 *	last edit-date: [Sat Sep 21 12:49:18 1996]
 *
 *	-hm	taking disklabel as an example
 *	-hm	stealing config routine from my good old cwish
 *	-hm	split off convert.c
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"

/*---------------------------------------------------------------------------*
 *	convert disklabel struct to internal menu disklabel struct
 *---------------------------------------------------------------------------*/
void
label_to_int(struct menu_disklabel *dls, struct disklabel *lp)
{
	dls->d_type = lp->d_type;
	dls->d_subtype = lp->d_subtype;
	strcpy(dls->d_typename, lp->d_typename);
	strcpy(dls->d_mpackname, lp->d_packname);
	dls->un_d_boot0[0] = '\0';
	dls->un_d_boot1[0] = '\0';
	dls->d_secsize = lp->d_secsize;
	dls->d_nsectors = lp->d_nsectors;
	dls->d_ntracks = lp->d_ntracks;
	dls->d_ncylinders = lp->d_ncylinders;
	dls->d_secpercyl = lp->d_secpercyl;
	dls->d_secperunit = lp->d_secperunit;
	dls->d_sparespertrack = lp->d_sparespertrack;
	dls->d_sparespercyl = lp->d_sparespercyl;
	dls->d_acylinders = lp->d_acylinders;
	dls->d_rpm = lp->d_rpm;
	dls->d_interleave = lp->d_interleave;
	dls->d_trackskew = lp->d_trackskew;
	dls->d_cylskew = lp->d_cylskew;
	dls->d_headswitch = lp->d_headswitch;
	dls->d_trkseek = lp->d_trkseek;
	dls->d_flags = lp->d_flags;
	dls->d_drivedata0 = lp->d_drivedata[0];
	dls->d_drivedata1 = lp->d_drivedata[1];
	dls->d_drivedata2 = lp->d_drivedata[2];
	dls->d_drivedata3 = lp->d_drivedata[3];
	dls->d_drivedata4 = lp->d_drivedata[4];
	dls->d_spare0 = lp->d_spare[0];
	dls->d_spare1 = lp->d_spare[1];
	dls->d_spare2 = lp->d_spare[2];
	dls->d_spare3 = lp->d_spare[3];
	dls->d_spare4 = lp->d_spare[4];
	dls->d_npartitions = lp->d_npartitions;
	dls->d_bbsize = lp->d_bbsize;
	dls->d_sbsize = lp->d_sbsize;

	part_to_int(&dls->part_a, &lp->d_partitions[0], lp);
	part_to_int(&dls->part_b, &lp->d_partitions[1], lp);	
	part_to_int(&dls->part_c, &lp->d_partitions[2], lp);	
	part_to_int(&dls->part_d, &lp->d_partitions[3], lp);	
	part_to_int(&dls->part_e, &lp->d_partitions[4], lp);	
	part_to_int(&dls->part_f, &lp->d_partitions[5], lp);	
	part_to_int(&dls->part_g, &lp->d_partitions[6], lp);	
	part_to_int(&dls->part_h, &lp->d_partitions[7], lp);	
}

/*---------------------------------------------------------------------------*
 *	convert partition struct to internal menu partition struct
 *---------------------------------------------------------------------------*/
void
part_to_int(menu_partition_t *mp, struct partition *pp, struct disklabel *lp)
{
	if(pp->p_size)
	{
		mp->p_size = pp->p_size;
		mp->p_offset = pp->p_offset;
		mp->p_fstype = pp->p_fstype;

		mp->p_beg = pp->p_offset / lp->d_secpercyl;
		mp->p_bega = pp->p_offset % lp->d_secpercyl;	
		mp->p_end = (pp->p_offset + pp->p_size + lp->d_secpercyl - 1) / lp->d_secpercyl - 1;
		mp->p_enda = pp->p_size % lp->d_secpercyl;

		if((pp->p_size * lp->d_secsize) >= 1048576)
			mp->p_mb = (pp->p_size * lp->d_secsize) / 1048576;
		else
			mp->p_mb = 0;
		
		switch(pp->p_fstype)
		{
			case FS_BSDFFS:
				mp->p_fsize = pp->p_fsize;
				mp->p_bsize = pp->p_fsize * pp->p_frag;
				mp->p_mcpg = pp->p_cpg;
				break;
			default:
				mp->p_fsize = 0;
				mp->p_bsize = 0;
				mp->p_mcpg = 0;
		}
	}
	else
	{
		mp->p_size = 0;
		mp->p_offset = 0;
		mp->p_fsize = 0;
		mp->p_fstype = 0;
		mp->p_bsize = 0;
		mp->p_mcpg = 0;
		mp->p_beg = 0;
		mp->p_bega = 0;
		mp->p_end = 0;
		mp->p_enda = 0;
		mp->p_mb = 0;		
	}
}
/*---------------------------------------------------------------------------*
 *	convert internal menu disklabel struct to disklabel struct
 *---------------------------------------------------------------------------*/
void
int_to_label(struct menu_disklabel *dls, struct disklabel *lp)
{
	lp->d_magic = DISKMAGIC;
	lp->d_type = dls->d_type;
	lp->d_subtype = dls->d_subtype;
	strcpy(lp->d_typename, dls->d_typename);
	strcpy(lp->d_packname, dls->d_mpackname); 
	lp->d_secsize = dls->d_secsize;
	lp->d_nsectors = dls->d_nsectors;
	lp->d_ntracks = dls->d_ntracks;
	lp->d_ncylinders = dls->d_ncylinders;
	lp->d_secpercyl = dls->d_secpercyl;
	lp->d_secperunit = dls->d_secperunit;
	lp->d_sparespertrack = dls->d_sparespertrack;
	lp->d_sparespercyl = dls->d_sparespercyl;
	lp->d_acylinders = dls->d_acylinders;
	lp->d_rpm = dls->d_rpm;
	lp->d_interleave = dls->d_interleave;
	lp->d_trackskew = dls->d_trackskew;
	lp->d_cylskew = dls->d_cylskew;
	lp->d_headswitch = dls->d_headswitch;
	lp->d_trkseek = dls->d_trkseek;
	lp->d_flags = dls->d_flags;
	lp->d_drivedata[0] = dls->d_drivedata0;
	lp->d_drivedata[1] = dls->d_drivedata1;
	lp->d_drivedata[2] = dls->d_drivedata2;
	lp->d_drivedata[3] = dls->d_drivedata3;
	lp->d_drivedata[4] = dls->d_drivedata4;
	lp->d_spare[0] = dls->d_spare0;
	lp->d_spare[1] = dls->d_spare1;
	lp->d_spare[2] = dls->d_spare2;
	lp->d_spare[3] = dls->d_spare3;
	lp->d_spare[4] = dls->d_spare4;
	lp->d_magic2 = DISKMAGIC;
	lp->d_npartitions = MAXPARTITIONS;
	lp->d_bbsize = dls->d_bbsize;
	lp->d_sbsize = dls->d_sbsize;
	int_to_part(&dls->part_a, &lp->d_partitions[0], lp);
	int_to_part(&dls->part_b, &lp->d_partitions[1], lp);	
	int_to_part(&dls->part_c, &lp->d_partitions[2], lp);	
	int_to_part(&dls->part_d, &lp->d_partitions[3], lp);	
	int_to_part(&dls->part_e, &lp->d_partitions[4], lp);	
	int_to_part(&dls->part_f, &lp->d_partitions[5], lp);	
	int_to_part(&dls->part_g, &lp->d_partitions[6], lp);	
	int_to_part(&dls->part_h, &lp->d_partitions[7], lp);	
}

/*---------------------------------------------------------------------------*
 *	convert internal menu partition struct to partiton struct
 *---------------------------------------------------------------------------*/
void
int_to_part(menu_partition_t *mp, struct partition *pp, struct disklabel *lp)
{
	if(mp->p_size)
	{
		pp->p_size = mp->p_size;
		pp->p_offset = mp->p_offset;
		pp->p_fstype = mp->p_fstype;

		switch(mp->p_fstype)
		{
			case FS_BSDFFS:
				pp->p_fsize = mp->p_fsize;
				if(mp->p_fsize)
					pp->p_frag = mp->p_bsize / mp->p_fsize;
				else
					pp->p_frag = 0;
				pp->p_cpg = mp->p_mcpg;
				break;
			default:
				pp->p_fsize = 0;
				pp->p_frag = 0;
				pp->p_cpg = 0;
		}
	}
	else
	{
		pp->p_size = 0;
		pp->p_offset = 0;
		pp->p_fstype = 0;
		pp->p_fsize = 0;
		pp->p_frag = 0;
		pp->p_cpg = 0;
	}
}

/*----------------------- E O F -------------------------------------------*/
