/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - curgets.c
 *	---------------------------------
 *
 *	last edit-date: [Sat Sep 21 12:50:52 1996]
 *
 *	-hm	needed error info
 *
 *---------------------------------------------------------------------------*/

#include <strings.h>
#include <stdlib.h>
#include <ncurses.h>			/* curses include file */

#include "control.h"

static char *press = "Press any key to continue ...";
static char *title = "ERROR MESSAGE";

/*---------------------------------------------------------------------------*
 *	curerr, print error message in a box
 *	len: the maximum number of chars in the text line (defines box size)
 *	message: error string pointer
 *---------------------------------------------------------------------------*/
void
curerr(int len, char *message)
{
	int y;
	int x;
	
	WINDOW *strwin;

	if (len >= COLS-5)
		len = COLS - 5;
	if (len < strlen(press))
		len = strlen(press);

	y = LINES/2;
	x = (COLS - len)/2;
 
	strwin = newwin(4, len+2, y, x);

	wborder(strwin, 0, 0, 0, 0, 0, 0, 0, 0);

	wstandout(strwin);
	mvwaddstr(strwin, 0, (len / 2) - (strlen(title) / 2), title);
	wstandend(strwin);

	mvwaddstr(strwin, 1, ((len / 2) - (strlen(message) / 2)) + 1, message);

	mvwaddstr(strwin, 2, ((len / 2) - (strlen(press) / 2)) + 1, press);

	wrefresh(strwin);

	noecho();
	crmode();

	wgetch(strwin);

	wclear(strwin);
	wrefresh(strwin);
	delwin(strwin);
}

/* EOF */
