/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - curgets.c
 *	---------------------------------
 *
 * 	based on: Curses getstring in a box
 *	          by Jim King (jek5036@ultb.isc.rit.edu)
 *
 *	last edit-date: [Sat Sep 21 12:52:21 1996]
 *
 *	-hm	integrating from menudir
 *
 *---------------------------------------------------------------------------*/

#include <strings.h>
#include <stdlib.h>
#include <ncurses.h>			/* curses include file */

#include "control.h"

#define BOXWIDTH 2
#define WINHEIGHT 3

/*---------------------------------------------------------------------------*
 *	curgeti - get integer from a curses box
 *	len: the maximum amount of digits to read in (defines box size)
 *	y, x: coordinates on the screen of the box's upper left-hand corner
 *	title: the title for the box
 *	hide: hide output, for passwd applications
 *	defnum: if not -1, num to display as default
 *---------------------------------------------------------------------------*/
int
curgeti(int len, int y, int x, char *title, int hide, int defnum)
{
	WINDOW *strwin;
	int c;
	char input[80];
	int pos;
	int curx;
	int bschar = erasechar();

	/* failsafe positioning */

	if((y + WINHEIGHT) >= LINES)
		y -= (y+WINHEIGHT+1) - LINES;

	if((x + len + BOXWIDTH) >= COLS)
		x -= (x+len+BOXWIDTH+1) - COLS;

	/* create window */
	
	strwin = newwin(WINHEIGHT, len+BOXWIDTH, y, x);
	wborder(strwin, 0, 0, 0, 0, 0, 0, 0, 0);

	wstandout(strwin);
	mvwaddstr(strwin, 0, ((len+BOXWIDTH) / 2) - (strlen(title) / 2), title);
	wstandend(strwin);

restart:
	curx = 1;

	if(defnum != -1)
	{
		sprintf(input, "%-d", defnum);
		mvwaddstr(strwin, 1, 1, input);
		curx = strlen(input) + 1;
	}
	else
	{
		for (pos = 1; pos < len; pos++)
			mvwaddch(strwin, 1, pos, ' ');
	}

	for (;;)
	{
		if(curx > len)
			wmove(strwin, 1, curx-1);
		else
			wmove(strwin, 1, curx);

		wrefresh(strwin);

		noecho();
		crmode();

		c = wgetch(strwin);

		if(c == bschar)
		{
			if (curx == 1)
				continue;
			mvwaddch(strwin, 1, --curx, ' ');
			input[curx-1] = '\0';
			continue;
		}

		switch(c)
		{

			case KEY_DL:	/* delete line */
				goto restart;
				break;

			case CR:
			case LF:	/* CR or LF */
				input[curx-1] = '\0';
				wclear(strwin);
				wrefresh(strwin);
				delwin(strwin);
				return(atoi(input));

			default:
				if (curx > len)
					break;

				if (c < '0' || c > '9')
					break;	/* only digits */

				if (!hide)
					mvwaddch(strwin, 1, curx++, c);
				else
					mvwaddch(strwin, 1, curx++, '*');

				input[curx-2] = c;
				break;
		}
	}
}
