/*
 * Copyright (c) 1987, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Copyright (C) 1996
 *	Hellmuth Michaelis.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Symmetric Computer Systems.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - diskutil.c
 *	----------------------------------
 *
 *	derived from the code of disklabel(8).
 *
 *	last edit-date: [Sat Sep 21 18:29:37 1996]
 *
 *	-hm	taking the needed code from disklabel ...
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"

/*---------------------------------------------------------------------------*
 *	build a list of all mounted drives
 *---------------------------------------------------------------------------*/	
int
get_mounted(void)
{
	static struct statfs *mntbuf;
	int count;
	int i;
	
	mountmax = 0;
	
	count = getmntinfo(&mntbuf, MNT_NOWAIT);

	if(!count)
		return(0);

	for(i = 0; i < count; i++)
	{
		if(mntbuf[i].f_type != MOUNT_UFS)
			continue;
			
		if((mountlist[mountmax] = malloc(sizeof(char *))) == NULL)
			exit(99);

		mountlist[mountmax] = mntbuf[i].f_mntfromname;
		mountmax++;
	}
	return(mountmax);
}

/*---------------------------------------------------------------------------*
 *	check if a device is mounted
 *---------------------------------------------------------------------------*/	
int
check_mounted(char *name)
{
	int i;
	char *ret = NULL;
	
	for(i = 0; i < mountmax; i++)
	{
		if((ret = strstr((char *)mountlist[i], name)) != NULL)
			break;
	}

	if(ret == NULL)
		return(FALSE);
	else
		return(TRUE);
}

/*---------------------------------------------------------------------------*
 *	read label from device user has entered and display it
 *---------------------------------------------------------------------------*/	
void
new_device(char *name)
{
	int f;
	char *p;
	char *ptr;
	
	/* construct raw devicename for c partition */

	if(*name != '/')
	{
		sprintf(specname, "%sr%s%c", _PATH_DEV, name, RAWPARTITION);
	}
	else
	{
		strcpy(specname, name);
	}

	/* try to open /dev/rXYNc */
	
	if((f = open(specname, O_RDONLY)) < 0)
	{
		Perror(specname);
		return;
	}

	/* extract the device name */

	if((p = rindex(specname, '/')) == NULL)
	{
		ptr = specname;
	}
	else
	{
		p++;

		if(*p == 'r')
			p++;

		ptr = p;
	}

	if((strncmp(ptr, DISK_WD, 2)) && (strncmp(ptr, DISK_SD, 2)))
	{
		char buf[256];
		sprintf(buf, "device [%s] not supported!", ptr);
		Error(buf);
		return;
	}

	strcpy(device, ptr);

	strcpy(specboot, ptr);	

	specboot[2] = '\0';
	
	p = device + 2;

	while(isdigit(*p))
		p++;

	*p = '\0';

	/* check if device is mounted */

	mountstate = check_mounted(device);
	
	/* read the label from device */
	
	lp = readlabel(f, 1 /* rflag */, lp);

	close(f);

	/* convert label to internal form */
	
	label_to_int(&dls, lp);	/* convert disklabel to internal struct */

	sprintf(dls.un_d_boot0, "%s/%s%s", _PATH_BOOTDIR, specboot, "boot");
	sprintf(dls.un_d_boot1, "%s/%s%s", _PATH_BOOTDIR, "boot", specboot);

	/* update display with new values */
	
	update_display();
}

/*---------------------------------------------------------------------------*
 *	print error message
 *---------------------------------------------------------------------------*/	
void
Error(char *str)
{
	curerr(strlen(str), str);
}

/*---------------------------------------------------------------------------*
 *	perror for curses display
 *---------------------------------------------------------------------------*/	
void
Perror(char *str)
{
	char buffer[256];
	sprintf(buffer, "%s, %s", str, sys_errlist[errno]);
	curerr(strlen(buffer), buffer);
}

/*---------------------------------------------------------------------------*
 *	print a warning for curses
 *---------------------------------------------------------------------------*/	
void
Warning(char *fmt, ...)
{
	char buffer[256];	
	va_list ap;

	sprintf(buffer, "Warning, ");
	va_start(ap, fmt);
	vsprintf(buffer, fmt, ap);
	va_end(ap);
	curerr(strlen(buffer), buffer);	
}

/*---------------------------------------------------------------------------*
 *	compute label checksum
 *---------------------------------------------------------------------------*/	
u_short
dkcksum(struct disklabel *lp)
{
	register u_short *start, *end;
	register u_short sum = 0;

	start = (u_short *)lp;
	end = (u_short *)&lp->d_partitions[lp->d_npartitions];
	while (start < end)
		sum ^= *start++;
	return (sum);
}

/*---------------------------------------------------------------------------*
 *	read a label from disk
 *---------------------------------------------------------------------------*/	
struct disklabel *
readlabel(int f, int rflag, struct disklabel *lp)
{
	struct disklabel lab;
	
	if (rflag)
	{
		if (read(f, bootarea, BBSIZE) < BBSIZE)
		{
			Perror(specname);
		}

		for (lp = (struct disklabel *)bootarea;
		    lp <= (struct disklabel *)(bootarea + BBSIZE - sizeof(*lp));
		    lp = (struct disklabel *)((char *)lp + 16))
		{
			if (lp->d_magic == DISKMAGIC &&
			    lp->d_magic2 == DISKMAGIC)
				break;
		}
		if (lp > (struct disklabel *)(bootarea+BBSIZE-sizeof(*lp)) ||
		    lp->d_magic != DISKMAGIC || lp->d_magic2 != DISKMAGIC ||
		    dkcksum(lp) != 0)
		{
			Error(
	"Bad pack magic number (label damaged or unlabeled), getting \"auto\"!");
			getdisktab("auto", NULL, lp);
			return(lp);
		}
	}
	else
	{
		lp = &lab;
		if (ioctl(f, DIOCGDINFO, lp) < 0)
		{
			Perror("ioctl DIOCGDINFO");
		}
	}
	return (lp);
}

/*---------------------------------------------------------------------------*
 *	check a label
 *---------------------------------------------------------------------------*/	
int
checklabel(struct disklabel *lp)
{
	register struct partition *pp;
	int i, errors = 0;
	char part;

	if (lp->d_secsize == 0)
	{
		char buffer[256];
		sprintf(buffer, "sector size %ld", lp->d_secsize);
		Error(buffer);
		return (1);
	}

	if (lp->d_nsectors == 0)
	{
		char buffer[256];		
		sprintf(buffer,"sectors/track %ld", lp->d_nsectors);
		Error(buffer);
		return (1);
	}

	if (lp->d_ntracks == 0)
	{
		char buffer[256];
		sprintf(buffer, "tracks/cylinder %ld", lp->d_ntracks);
		Error(buffer);
		return (1);
	}

	if  (lp->d_ncylinders == 0)
	{
		char buffer[256];		
		sprintf(buffer, "cylinders/unit %ld", lp->d_ncylinders);
		errors++;
		Error(buffer);
	}

	if (lp->d_rpm == 0)
		Warning("revolutions/minute %d", lp->d_rpm);

	if (lp->d_secpercyl == 0)
		lp->d_secpercyl = lp->d_nsectors * lp->d_ntracks;

	if (lp->d_secperunit == 0)
		lp->d_secperunit = lp->d_secpercyl * lp->d_ncylinders;

	if (lp->d_bbsize == 0)
	{
		char buffer[256];
		sprintf(buffer, "boot block size %ld", lp->d_bbsize);
		errors++;
		Error(buffer);
	}
	else if (lp->d_bbsize % lp->d_secsize)
	{
		Warning("boot block size %% sector-size != 0");
	}
	
	if (lp->d_sbsize == 0)
	{
		char buffer[256];
		sprintf(buffer, "super block size %ld", lp->d_sbsize);
		errors++;
		Error(buffer);
	}
	else if (lp->d_sbsize % lp->d_secsize)
	{
		Warning("super block size %% sector-size != 0");
	}

	if (lp->d_npartitions > MAXPARTITIONS)
	{
		Warning("number of partitions (%d) > MAXPARTITIONS (%d)",
		    lp->d_npartitions, MAXPARTITIONS);
	}
	
	for (i = 0; i < lp->d_npartitions; i++)
	{
		part = 'a' + i;
		pp = &lp->d_partitions[i];

		if (pp->p_size == 0 && pp->p_offset != 0)
			Warning("partition %c: size 0, but offset %d",
			    part, pp->p_offset);
/* #ifdef notdef */
		if (pp->p_size % lp->d_secpercyl)
			Warning("partition %c: size %% cylinder-size != 0",
			    part);
		if (pp->p_offset % lp->d_secpercyl)
			Warning("partition %c: offset %% cylinder-size != 0",
			    part);
/* #endif */

		if (pp->p_offset > lp->d_secperunit)
		{
			char buffer[256];
			sprintf(buffer, 
				"partition %c: offset past end of unit", part);
			Error(buffer);
			errors++;
		}

		if (pp->p_offset + pp->p_size > lp->d_secperunit)
		{
			char buffer[256];
			sprintf(buffer,
				"partition %c: partition extends past end of unit\n",
			    part);
			Error(buffer);
			errors++;
		}
	}

	for (; i < MAXPARTITIONS; i++)
	{
		part = 'a' + i;
		pp = &lp->d_partitions[i];

		if (pp->p_size || pp->p_offset)
			Warning("unused partition %c: size %d offset %d",
			    'a' + i, pp->p_size, pp->p_offset);
	}
	return (errors);
}

static char *
skip(register char *cp)
{

	while (*cp != '\0' && isspace(*cp))
		cp++;
	if (*cp == '\0' || *cp == '#')
		return ((char *)NULL);
	return (cp);
}

static char *
word(register char *cp)
{
	register char c;

	while (*cp != '\0' && !isspace(*cp) && *cp != '#')
		cp++;
	if ((c = *cp) != '\0') {
		*cp++ = '\0';
		if (c != '#')
			return (skip(cp));
	}
	return ((char *)NULL);
}

#define	streq(a,b)	(strcmp(a,b) == 0)

/*---------------------------------------------------------------------------*
 * Read an ascii label in from fd f, in the same format as that
 * put out by putasciilabel(), and fill in lp.
 *---------------------------------------------------------------------------*/	
int
getasciilabel(FILE *f, struct disklabel *lp)
{
	register char **cpp, *cp;
	register struct partition *pp;
	char *tp, *s, line[BUFSIZ];
	int v, lineno = 0, errors = 0;
	char buffer[256];
	
	lp->d_bbsize = BBSIZE;				/* XXX */
	lp->d_sbsize = SBSIZE;				/* XXX */
	while (fgets(line, sizeof(line) - 1, f)) {
		lineno++;
		if ((cp = index(line,'\n')) != 0)
			*cp = '\0';
		cp = skip(line);
		if (cp == NULL)
			continue;
		tp = index(cp, ':');
		if (tp == NULL)
		{
			sprintf(buffer, "line %d: syntax error\n", lineno);
			errors++;
			Error(buffer);
			continue;
		}
		*tp++ = '\0', tp = skip(tp);
		if (streq(cp, "type")) {
			if (tp == NULL)
				tp = "unknown";
			cpp = dktypenames;
			for (; cpp < &dktypenames[DKMAXTYPES]; cpp++)
				if ((s = *cpp) && streq(s, tp)) {
					lp->d_type = cpp - dktypenames;
					goto next;
				}
			v = atoi(tp);
			if ((unsigned)v >= DKMAXTYPES)
			{
				Warning("line %d:%s %d\n", lineno,
				    "Warning, unknown disk type", v);
			}
			lp->d_type = v;
			continue;
		}
		if (streq(cp, "flags")) {
			for (v = 0; (cp = tp) && *cp != '\0';) {
				tp = word(cp);
				if (streq(cp, "removeable"))
					v |= D_REMOVABLE;
				else if (streq(cp, "ecc"))
					v |= D_ECC;
				else if (streq(cp, "badsect"))
					v |= D_BADSECT;
				else
				{
					sprintf(buffer,
					    "line %d: %s: bad flag\n",
					    lineno, cp);
					errors++;
					Error(buffer);
				}
			}
			lp->d_flags = v;
			continue;
		}
		if (streq(cp, "drivedata")) {
			register int i;

			for (i = 0; (cp = tp) && *cp != '\0' && i < NDDATA;) {
				lp->d_drivedata[i++] = atoi(cp);
				tp = word(cp);
			}
			continue;
		}
		if (sscanf(cp, "%d partitions", &v) == 1) {
			if (v == 0 || (unsigned)v > MAXPARTITIONS)
			{
				sprintf(buffer,
				    "line %d: bad # of partitions\n", lineno);
				lp->d_npartitions = MAXPARTITIONS;
				Error(buffer);
				errors++;
			} else
				lp->d_npartitions = v;
			continue;
		}
		if (tp == NULL)
			tp = "";
		if (streq(cp, "disk")) {
			strncpy(lp->d_typename, tp, sizeof (lp->d_typename));
			continue;
		}
		if (streq(cp, "label")) {
			strncpy(lp->d_packname, tp, sizeof (lp->d_packname));
			continue;
		}
		if (streq(cp, "bytes/sector")) {
			v = atoi(tp);
			if (v <= 0 || (v % 512) != 0)
			{
				sprintf(buffer,
				    "line %d: %s: bad sector size\n",
				    lineno, tp);
				errors++;
				Error(buffer);
			}
			else
			{
				lp->d_secsize = v;
			}
			continue;
		}
		if (streq(cp, "sectors/track")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_nsectors = v;
			continue;
		}
		if (streq(cp, "sectors/cylinder")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_secpercyl = v;
			continue;
		}
		if (streq(cp, "tracks/cylinder")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_ntracks = v;
			continue;
		}
		if (streq(cp, "cylinders")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_ncylinders = v;
			continue;
		}
		if (streq(cp, "sectors/unit")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_secperunit = v;
			continue;
		}
		if (streq(cp, "rpm")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_rpm = v;
			continue;
		}
		if (streq(cp, "interleave")) {
			v = atoi(tp);
			if (v <= 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_interleave = v;
			continue;
		}
		if (streq(cp, "trackskew")) {
			v = atoi(tp);
			if (v < 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_trackskew = v;
			continue;
		}
		if (streq(cp, "cylinderskew")) {
			v = atoi(tp);
			if (v < 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_cylskew = v;
			continue;
		}
		if (streq(cp, "headswitch")) {
			v = atoi(tp);
			if (v < 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_headswitch = v;
			continue;
		}
		if (streq(cp, "track-to-track seek")) {
			v = atoi(tp);
			if (v < 0)
			{
				sprintf(buffer, "line %d: %s: bad %s\n",
				    lineno, tp, cp);
				errors++;
				Error(buffer);
			} else
				lp->d_trkseek = v;
			continue;
		}
		if ('a' <= *cp && *cp <= 'z' && cp[1] == '\0')
		{
			unsigned part = *cp - 'a';

			if (part > lp->d_npartitions)
			{
				sprintf(buffer,
				    "line %d: bad partition name\n", lineno);
				errors++;
				Error(buffer);
				continue;
			}
			pp = &lp->d_partitions[part];
#define NXTNUM(n) { \
	if (tp == NULL) { \
		sprintf(buffer, "line %d: too few numeric fields\n", lineno); \
		errors++; \
		Error(buffer); \
		break; \
	} else { \
		cp = tp, tp = word(cp); \
		if (tp == NULL) \
			tp = cp; \
		(n) = atoi(cp); \
	} \
     }

			NXTNUM(v);
			if (v < 0)
			{
				sprintf(buffer,
				    "line %d: %s: bad partition size\n",
				    lineno, cp);
				errors++;
				Error(buffer);
			} else
				pp->p_size = v;
			NXTNUM(v);
			if (v < 0)
			{
				sprintf(buffer,
				    "line %d: %s: bad partition offset\n",
				    lineno, cp);
				errors++;
				Error(buffer);
			} else
				pp->p_offset = v;
			cp = tp, tp = word(cp);
			cpp = fstypenames;
			for (; cpp < &fstypenames[FSMAXTYPES]; cpp++)
				if ((s = *cpp) && streq(s, cp)) {
					pp->p_fstype = cpp - fstypenames;
					goto gottype;
				}
			if (isdigit(*cp))
				v = atoi(cp);
			else
				v = FSMAXTYPES;
			if ((unsigned)v >= FSMAXTYPES)
			{
				Warning("line %d: %s %s\n", lineno,
				    "Warning, unknown filesystem type", cp);
				v = FS_UNUSED;
			}
			pp->p_fstype = v;
	gottype:

			switch (pp->p_fstype) {

			case FS_UNUSED:				/* XXX */
				NXTNUM(pp->p_fsize);
				if (pp->p_fsize == 0)
					break;
				NXTNUM(v);
				pp->p_frag = v / pp->p_fsize;
				break;

			case FS_BSDFFS:
				NXTNUM(pp->p_fsize);
				if (pp->p_fsize == 0)
					break;
				NXTNUM(v);
				pp->p_frag = v / pp->p_fsize;
				NXTNUM(pp->p_cpg);
				break;

			case FS_BSDLFS:
				NXTNUM(pp->p_fsize);
				if (pp->p_fsize == 0)
					break;
				NXTNUM(v);
				pp->p_frag = v / pp->p_fsize;
				NXTNUM(pp->p_cpg);
				break;

			default:
				break;
			}
			continue;
		}
		sprintf(buffer, "line %d: %s: Unknown disklabel field\n",
		    lineno, cp);
		errors++;
		Error(buffer);
	next:
		;
	}
	errors += checklabel(lp);
	return (errors == 0);
}

/*---------------------------------------------------------------------------*
 * Write an ascii label in from fd f, in the same format as that
 * what can be read by getasciilabel()
 *---------------------------------------------------------------------------*/	
void
putasciilabel(FILE *f, register struct disklabel *lp)
{
	register int i, j;
	register struct partition *pp;

	fprintf(f, "# %s:\n", specname);
	if ((unsigned) lp->d_type < DKMAXTYPES)
		fprintf(f, "type: %s\n", dktypenames[lp->d_type]);
	else
		fprintf(f, "type: %d\n", lp->d_type);
	fprintf(f, "disk: %.*s\n", (int)sizeof(lp->d_typename),
		lp->d_typename);
	fprintf(f, "label: %.*s\n", (int)sizeof(lp->d_packname),
		lp->d_packname);
	fprintf(f, "flags:");
	if (lp->d_flags & D_REMOVABLE)
		fprintf(f, " removeable");
	if (lp->d_flags & D_ECC)
		fprintf(f, " ecc");
	if (lp->d_flags & D_BADSECT)
		fprintf(f, " badsect");
	fprintf(f, "\n");
	fprintf(f, "bytes/sector: %ld\n", lp->d_secsize);
	fprintf(f, "sectors/track: %ld\n", lp->d_nsectors);
	fprintf(f, "tracks/cylinder: %ld\n", lp->d_ntracks);
	fprintf(f, "sectors/cylinder: %ld\n", lp->d_secpercyl);
	fprintf(f, "cylinders: %ld\n", lp->d_ncylinders);
	fprintf(f, "sectors/unit: %ld\n", lp->d_secperunit);
	fprintf(f, "rpm: %d\n", lp->d_rpm);
	fprintf(f, "interleave: %d\n", lp->d_interleave);
	fprintf(f, "trackskew: %d\n", lp->d_trackskew);
	fprintf(f, "cylinderskew: %d\n", lp->d_cylskew);
	fprintf(f, "headswitch: %ld\t\t# milliseconds\n", lp->d_headswitch);
	fprintf(f, "track-to-track seek: %ld\t# milliseconds\n",
		lp->d_trkseek);
	fprintf(f, "drivedata: ");
	for (i = NDDATA - 1; i >= 0; i--)
		if (lp->d_drivedata[i])
			break;
	if (i < 0)
		i = 0;
	for (j = 0; j <= i; j++)
		fprintf(f, "%ld ", lp->d_drivedata[j]);
	fprintf(f, "\n\n%d partitions:\n", lp->d_npartitions);
	fprintf(f,
	    "#        size   offset    fstype   [fsize bsize bps/cpg]\n");
	pp = lp->d_partitions;
	for (i = 0; i < lp->d_npartitions; i++, pp++) {
		if (pp->p_size) {
			fprintf(f, "  %c: %8ld %8ld  ", 'a' + i,
			   pp->p_size, pp->p_offset);
			if ((unsigned) pp->p_fstype < FSMAXTYPES)
				fprintf(f, "%8.8s", fstypenames[pp->p_fstype]);
			else
				fprintf(f, "%8d", pp->p_fstype);
			switch (pp->p_fstype) {

			case FS_UNUSED:				/* XXX */
				fprintf(f, "    %5ld %5ld %5.5s ",
				    pp->p_fsize, pp->p_fsize * pp->p_frag, "");
				break;

			case FS_BSDFFS:
				fprintf(f, "    %5ld %5ld %5d ",
				    pp->p_fsize, pp->p_fsize * pp->p_frag,
				    pp->p_cpg);
				break;

			case FS_BSDLFS:
				fprintf(f, "    %5ld %5ld %5d",
				    pp->p_fsize, pp->p_fsize * pp->p_frag,
				    pp->p_cpg);
				break;

			default:
				fprintf(f, "%20.20s", "");
				break;
			}
			fprintf(f, "\t# (Cyl. %4ld",
			    pp->p_offset / lp->d_secpercyl);
			if (pp->p_offset % lp->d_secpercyl)
			    putc('*', f);
			else
			    putc(' ', f);
			fprintf(f, "- %ld",
			    (pp->p_offset +
			    pp->p_size + lp->d_secpercyl - 1) /
			    lp->d_secpercyl - 1);
			if (pp->p_size % lp->d_secpercyl)
			    putc('*', f);
			fprintf(f, ")\n");
		}
	}
	fflush(f);
}

/*---------------------------------------------------------------------------*
 * When operating on a "virgin" disk, try getting an initial label
 * from the associated device driver.  This might work for all device
 * drivers that are able to fetch some initial device parameters
 * without even having access to a (BSD) disklabel, like SCSI disks,
 * most IDE drives, or vn devices. The device name must be given in
 * its "canonical" form.
 *---------------------------------------------------------------------------*/	
static struct disklabel *
getvirginlabel(void)
{
	static struct disklabel lab;
	char namebuf[BBSIZE];
	int f;

	(void)snprintf(namebuf, BBSIZE, "%sr%s", _PATH_DEV, device);

	if ((f = open(namebuf, O_RDONLY, 0)) == -1)
	{
		Perror("getvirginlabel, open()");
		return 0;
	}

	if (ioctl(f, DIOCGDINFO, &lab) < 0)
	{
		Perror("ioctl DIOCGDINFO");
		close(f);
		return 0;
	}
	
	close(f);

	/* insert reasonable defaults where necessary */

	if((lab.d_npartitions < 8) || (lab.d_npartitions > MAXPARTITIONS))
		lab.d_npartitions = MAXPARTITIONS;
		
	if (lab.d_bbsize == 0)
		lab.d_bbsize = BBSIZE;
		
	if (lab.d_sbsize == 0)
		lab.d_sbsize = SBSIZE;
		
	if (lab.d_rpm == 0)
		lab.d_rpm = 3600;

	if (lab.d_interleave == 0)
		lab.d_interleave = 1;

	return (&lab);
}

/*---------------------------------------------------------------------------*
 * Construct a prototype disklabel from /etc/disktab.  As a side
 * effect, set the names of the primary and secondary boot files
 * if specified.
 *---------------------------------------------------------------------------*/	
void
getdisktab(char *type, char *name, register struct disklabel *lp)
{
	register struct disklabel *dp;

	if (strcmp(type, "auto") == 0)
		dp = getvirginlabel();
	else
		dp = getdiskbyname(type);

	if (dp == NULL)
	{
		char buffer[256];
		sprintf(buffer, "%s: unknown disk type!", type);
		Error(buffer);
		return;
	}

	*lp = *dp;

#if NUMBOOT > 0
	/*
	 * Set bootstrap name(s).
	 * 1. If set from command line, use those,
	 * 2. otherwise, check if disktab specifies them (b0 or b1),
	 * 3. otherwise, makebootarea() will choose ones based on the name
	 *    of the disk special file. E.g. /dev/ra0 -> raboot, bootra
	 */
	if (!xxboot && lp->d_boot0)
	{
		if (*lp->d_boot0 != '/')
		{
			(void)sprintf(boot0, "%s/%s",
				      _PATH_BOOTDIR, lp->d_boot0);
		}
		else
		{
			(void)strcpy(boot0, lp->d_boot0);
		}
		xxboot = boot0;
	}

#if NUMBOOT > 1
	if (!bootxx && lp->d_boot1)
	{
		if (*lp->d_boot1 != '/')
		{
			(void)sprintf(boot1, "%s/%s",
				      _PATH_BOOTDIR, lp->d_boot1);
		}
		else
		{
			(void)strcpy(boot1, lp->d_boot1);
		}
		bootxx = boot1;
	}
#endif

#endif
	/* d_packname is union d_boot[01], so zero */

	bzero(lp->d_packname, sizeof(lp->d_packname));

	if (name)
	{
		(void)strncpy(lp->d_packname, name, sizeof(lp->d_packname));
	}
}


/*
 * If we are installing a boot program that doesn't fit in d_bbsize
 * we need to mark those partitions that the boot overflows into.
 * This allows newfs to prevent creation of a filesystem where it might
 * clobber bootstrap code.
 */
void
setbootflag(lp)
	register struct disklabel *lp;
{
	register struct partition *pp;
	int i, errors = 0;
	char part;
	u_long boffset;

	if (bootbuf == 0)
		return;
	boffset = bootsize / lp->d_secsize;
	for (i = 0; i < lp->d_npartitions; i++) {
		part = 'a' + i;
		pp = &lp->d_partitions[i];
		if (pp->p_size == 0)
			continue;
		if (boffset <= pp->p_offset) {
			if (pp->p_fstype == FS_BOOT)
				pp->p_fstype = FS_UNUSED;
		} else if (pp->p_fstype != FS_BOOT) {
			if (pp->p_fstype != FS_UNUSED)
			{
				char buffer[256];			
				sprintf(buffer,
					"boot overlaps used partition %c",
					part);
				errors++;
				Error(buffer);
			} else {
				pp->p_fstype = FS_BOOT;
				Warning("boot overlaps partition %c, %s",
					part, "marked as FS_BOOT");
			}
		}
	}
	if (errors)
	{
		char buffer[256];
		sprintf(buffer, "Cannot install boot program!");
	}
}

/*---------------------------------------------------------------------------*
 *	write label to disk
 *---------------------------------------------------------------------------*/	
int
writelabel(int f, char *boot, register struct disklabel *lp)
{
	int flag;

	setbootflag(lp);
	lp->d_magic = DISKMAGIC;
	lp->d_magic2 = DISKMAGIC;
	lp->d_checksum = 0;
	lp->d_checksum = dkcksum(lp);

	/*
	 * First set the kernel disk label,
	 * then write a label to the raw disk.
	 * If the SDINFO ioctl fails because it is unimplemented,
	 * keep going; otherwise, the kernel consistency checks
	 * may prevent us from changing the current (in-core)
	 * label.
	 */

	if (ioctl(f, DIOCSDINFO, lp) < 0 && errno != ENODEV && errno != ENOTTY)
	{
		Perror("ioctl DIOCSDINFO");
		return (1);
	}

	(void)lseek(f, (off_t)0, SEEK_SET);

	/*
	 * write enable label sector before write (if necessary),
	 * disable after writing.
	 */

	flag = 1;

	if (ioctl(f, DIOCWLABEL, &flag) < 0)
		perror("ioctl DIOCWLABEL");
	if (write(f, boot, lp->d_bbsize) != lp->d_bbsize) {
		perror("write");
		return (1);
	}

	/*
	 * Output the remainder of the disklabel
	 */
	if (bootbuf && write(f, bootbuf, bootsize) != bootsize) {
		perror("write");
		return(1);
	}

	flag = 0;
	(void) ioctl(f, DIOCWLABEL, &flag);
	return (0);
}

/*---------------------------------------------------------------------------*
 * Construct a bootarea (d_bbsize bytes) in the specified buffer ``boot''
 * Returns a pointer to the disklabel portion of the bootarea.
 *---------------------------------------------------------------------------*/	
struct disklabel *
makebootarea(char *boot, register struct disklabel *dp, int f)
{
	struct disklabel *lp;
	register char *p;
	int b;
	char *tmpbuf;
	int i, found;

	if (dp->d_secsize == 0) {
		dp->d_secsize = DEV_BSIZE;
		dp->d_bbsize = BBSIZE;
	}
	lp = (struct disklabel *)
		(boot + (LABELSECTOR * dp->d_secsize) + LABELOFFSET);
	bzero((char *)lp, sizeof *lp);

	/*
	 * If we are not installing a boot program but we are installing a
	 * label on disk then we must read the current bootarea so we don't
	 * clobber the existing boot.
	 */
	if (!installboot)
	{
		if (read(f, boot, BBSIZE) < BBSIZE)
			Perror(specname);
		bzero((char *)lp, sizeof *lp);
		return (lp);
	}
	/*
	 * We are installing a boot program.  Determine the name(s) and
	 * read them into the appropriate places in the boot area.
	 */
	if(!xxboot)
		xxboot = dls.un_d_boot0;
		
	if(!bootxx)
		bootxx = dls.un_d_boot1;		

	/*
	 * 2. Two-piece bootstrap
	 *	up to d_secsize bytes of ``xxboot'' go in first d_secsize
	 *	bytes of bootarea, remaining d_bbsize-d_secsize filled
	 *	from ``bootxx''.
	 */
	b = open(xxboot, O_RDONLY);

	if (b < 0)
		Perror(xxboot);

	/*
	 * XXX Botch alert.
	 * The i386 has the so-called fdisk table embedded into the
	 * primary bootstrap.  We take care to not clobber it, but
	 * only if it does already contain some data.  (Otherwise,
	 * the xxboot provides a template.)
	 */

	if ((tmpbuf = (char *)malloc((int)dp->d_secsize)) == 0)
		Perror(xxboot);

	memcpy((void *)tmpbuf, (void *)boot, (int)dp->d_secsize);

	if (read(b, boot, (int)dp->d_secsize) < 0)
		Perror(xxboot);

	(void)close(b);

	for (i = DOSPARTOFF, found = 0;
	     !found && i < DOSPARTOFF + NDOSPART*sizeof(struct dos_partition);
	     i++)
	{
		found = tmpbuf[i] != 0;
	}
	
	if (found)
	{
		memcpy((void *)&boot[DOSPARTOFF],
		       (void *)&tmpbuf[DOSPARTOFF],
		       NDOSPART * sizeof(struct dos_partition));
	}
	
	free(tmpbuf);

	b = open(bootxx, O_RDONLY);

	if (b < 0)
		Perror(bootxx);

	if (read(b, &boot[dp->d_secsize],
		 (int)(dp->d_bbsize-dp->d_secsize)) < 0)
	{
		Perror(bootxx);
	}

	(void)close(b);

	/*
	 * Make sure no part of the bootstrap is written in the area
	 * reserved for the label.
	 */
	for (p = (char *)lp; p < (char *)lp + sizeof(struct disklabel); p++)
	{
		if (*p)
		{
			Error("Bootstrap doesn't leave room for disk label!");
		}
	}
	return (lp);
}

/* EOF */
