/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - display.c
 *	---------------------------------
 *
 *	last edit-date: [Sat Sep 21 12:47:29 1996]
 *
 *	-hm	taking disklabel as an example
 *	-hm	stealing config routine from my good old cwish
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"
#include "control.h"

/*---------------------------------------------------------------------------*
 *	display an item
 *---------------------------------------------------------------------------*/
void
display_item(struct item *ip, int inv)
{
	int i;
	
	standend();
	
	mvaddstr(ip->row, ip->col, ip->name);

	i = ip->length;

	while(i--)
		addch(' ');

	mvaddstr(ip->row, ip->col, ip->name);

	if(inv)
		standout();
	else
		standend();

	switch (ip->type & ITEM_MASK)
	{
		case ITEM_ONOFF:
			if(*(ip->value))
				addnstr("YES", ip->length);
			else
				addnstr(" NO", ip->length);
			break;

		case ITEM_INDIC:
			if(*(ip->value))
				addstr("*");
			else
				addstr(" ");			
			break;

		case ITEM_STATE:
			if(*(ip->value))
				addstr("mounted");
			else
				addstr("unmounted");			
			break;

		case ITEM_DEVICE:
			if(*((char *) ip->value))
				addnstr((char *) ip->value, ip->length);
			else
				addnstr("[Not set]", ip->length);
			break;

		case ITEM_COMMAND:
			addnstr("command menu", ip->length);
			break;

		case ITEM_STRING:
			if(*((char *) ip->value))
				addnstr((char *) ip->value, ip->length);
			else
				addnstr("[Not set]", ip->length);
			break;

		case ITEM_FSTYPE:
			if((int) *(ip->value) < FSMAXTYPES)
				addnstr(fstypenames[*(ip->value)], ip->length);
			else
				printw("%d", (int) *(ip->value));
			break;

		case ITEM_DRVTYPE:
			if((int) *(ip->value) < DKMAXTYPES)
				addnstr(dktypenames[*(ip->value)], ip->length);
			else
				printw("%d", (int) *(ip->value));
			break;

		case ITEM_INT:
		case ITEM_BPS:
		case ITEM_FSIZE:
		case ITEM_BSIZE:
		case ITEM_PSZ:
		case ITEM_PBC:
		case ITEM_PEC:
		case ITEM_PMB:
			printw("%-*d", ip->length, (int) *(ip->value));
			break;

		case ITEM_CHAR:
			if ((char) *(ip->value))
				addch((char) *(ip->value));
			else
				addstr("[Not set]");
			break;


			printw("%d", (int) *(ip->value));
			break;


	}
	standend();
}

/*---------------------------------------------------------------------------*
 *	print a horizontal line
 *---------------------------------------------------------------------------*/
void
separate(int row)
{
	int i = COLS-1;

	move(row, 0);

	attron(A_ALTCHARSET);

	while (i--)
		addch(ACS_HLINE);

	attroff(A_ALTCHARSET);
}

/*---------------------------------------------------------------------------*
 *	print headerline
 *---------------------------------------------------------------------------*/
void
hdrline(int row)
{
	char buf[256];
	
	int i = COLS-1;
	int j;
	
	move(row, 0);

	standout();
	attron(A_ALTCHARSET);
	for(j=5; j>0; j--, i--)
		addch(ACS_HLINE);
	attroff(A_ALTCHARSET);

	standout();
	sprintf(buf, " disk partition editor ");
	i -= strlen(buf);
	addstr(buf);

	attron(A_ALTCHARSET);	
	for(j=20; j>0; j--, i--)
		addch(ACS_HLINE);
	attroff(A_ALTCHARSET);	

	standout();
	sprintf(buf, " version %s.%s (%s) ", VERS, REV, __DATE__);
	i -= strlen(buf);
	addstr(buf);

	attron(A_ALTCHARSET);	
	while (i--)
		addch(ACS_HLINE);
	attroff(A_ALTCHARSET);
	standend();
}

/*---------------------------------------------------------------------------*
 *	update the whole diplay
 *---------------------------------------------------------------------------*/
void update_display(void)
{
	struct item *ip;

	/* set item pointer */
	
	ip = FIRST_I;

	do
		display_item(ip, 0);
	while ((ip = ip->next) != UFIRST_I);
	update_upperdisplay();
	update_lowerdisplay();	
}

/*---------------------------------------------------------------------------*
 *	update the disk geometry display
 *---------------------------------------------------------------------------*/
void update_upperdisplay(void)
{
	struct item *ip;

        dls.d_secpercyl = dls.d_nsectors * dls.d_ntracks;

        dls.d_secperunit = dls.d_secpercyl * dls.d_ncylinders;
	
	/* set item pointer */
	
	ip = UFIRST_I;

	do
		display_item(ip, 0);
	while ((ip = ip->next) != ULAST_I);
}

/*---------------------------------------------------------------------------*
 *	update the partition display
 *---------------------------------------------------------------------------*/
void update_lowerdisplay(void)
{
	struct item *ip;
	register menu_partition_t *mpp = &dls.part_a;
	int size = 0;
	int bcyl = 0;
	
	for(mpp = &dls.part_a; mpp <= &dls.part_h; mpp++)
	{
		if(mpp->p_fstype != FS_UNUSED &&
		   mpp != &dls.part_c && mpp != &dls.part_d)
		{
			mpp->p_beg = bcyl;
			bcyl = mpp->p_end;
			
			mpp->p_offset = size;
			size += mpp->p_size;
		}
	}
			
	/* set item pointer */
	
	ip = LFIRST_I;
	
	do
		display_item(ip, 0);
	while ((ip = ip->next) != FIRST_I);
}

/*----------------------- E O F -------------------------------------------*/
