/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - edit.c
 *	------------------------------
 *
 *	last edit-date: [Sat Sep 21 11:26:02 1996]
 *
 *	-hm	taking disklabel as an example
 *	-hm	stealing config routine from my good old cwish
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"
#include "menus.h"

static int first_row;
static int last_row;

static void changed_psz(register menu_partition_t *mpp);
static void changed_pbc(register menu_partition_t *mpp);
static void changed_pec(register menu_partition_t *mpp);
static void changed_pmb(register menu_partition_t *mpp);

/*---------------------------------------------------------------------------*
 *	edit an item
 *---------------------------------------------------------------------------*/
struct item *
edit_item(struct item *ip, int c)
{
	char newdev[256];
	int i, j;
	int r;
	int diff;
	struct item *ipsave;
	register menu_partition_t *mpp = &dls.part_a;
	
	if(ip->type & ITEM_SPEC)
		return(ip);
	
	switch (c)
	{
		case CR:

			standend();

			mvaddstr(ip->row, ip->col, ip->name);

			i = ip->length;
			while(i--)
			{
				addch(' ');
			}
		
			switch(ip->type)
			{
				case ITEM_COMMAND:
					i = menubar(ip->row+1,
						ip->col+strlen(ip->name),
						commands, 1,
						"command", 0);
					execute(i);
					break;					


				case ITEM_DEVICE: /* which device we're on */
					curgets(newdev,
						ip->length,
						ip->row+1,
						ip->col+strlen(ip->name),
						"device", 0,
						(char *) ip->value);
					new_device(newdev);
					break;					

				case ITEM_INT:
					*((int *)ip->value) = curgeti(
							ip->length,
							ip->row+1,
							ip->col+strlen(ip->name),
							"number", 0,
							(int) *ip->value);
					break;					

				case ITEM_STRING:
					curgets((char *) ip->value,
						ip->length,
						ip->row+1,
						ip->col+strlen(ip->name),
						"string", 0,
						(char *) ip->value);
					break;					

				case ITEM_DRVTYPE:
					*(ip->value) = (menubar(
							ip->row+1,
							ip->col+strlen(ip->name),
							dktypenames, 1,
							"type", *(ip->value))
							) - 1;
					break;					

				case ITEM_FSTYPE:
					*(ip->value) = (menubar(
							ip->row+1,
							ip->col+strlen(ip->name),
							fstypenames, 1,
							"fstype",
							*(ip->value))
							) - 1;
					break;					

				case ITEM_BPS:
					for(i=0;;i++)
					{
						if(*(ip->value) == bps_val[i])
							break;
					}		
					*(ip->value) = bps_val[(menubar(
							ip->row+1,
							ip->col+strlen(ip->name),
							bps_txt, 1,
							"bps",i)) - 1];
					break;					

				case ITEM_FSIZE:
					for(i=0;;i++)
					{
						if(*(ip->value) == fsize_val[i])
							break;
					}		

					j = menubar(ip->row+1,
							ip->col+strlen(ip->name),
							fsize_txt, 1,
							"fsize", i) - 1;
					*(ip->value) = fsize_val[j];
					*(ip->next->value) = bsize_val[j];
					break;					

				case ITEM_BSIZE:
					for(i=0;;i++)
					{
						if(*(ip->value) == bsize_val[i])
							break;
					}		
					j = menubar(ip->row+1,
							ip->col+strlen(ip->name),
							bsize_txt, 1,
							"bsize",i) - 1;
					*(ip->value) = bsize_val[j];
					*(ip->prev->value) = fsize_val[j];
					break;					

				case ITEM_PSZ:
					*((int *)ip->value) = curgeti(
							ip->length,
							ip->row+1,
							ip->col+strlen(ip->name),
							"number", 0,
							(int) *ip->value);

					mpp += (ip->row - ROW_PA);

					changed_psz(mpp);

					break;					

				case ITEM_PBC:
					*((int *)ip->value) = curgeti(
							ip->length,
							ip->row+1,
							ip->col+strlen(ip->name),
							"number", 0,
							(int) *ip->value);

					mpp += (ip->row - ROW_PA);

					changed_pbc(mpp);
					
					break;					

				case ITEM_PEC:
					*((int *)ip->value) = curgeti(
							ip->length,
							ip->row+1,
							ip->col+strlen(ip->name),
							"number", 0,
							(int) *ip->value);

					mpp += (ip->row - ROW_PA);

					changed_pec(mpp);

					break;					

				case ITEM_PMB:
					*((int *)ip->value) = curgeti(
							ip->length,
							ip->row+1,
							ip->col+strlen(ip->name),
							"int", 0,
							(int) *ip->value);

					mpp += (ip->row - ROW_PA);

					changed_pmb(mpp);

					break;					
			}

			if(ip >= UFIRST_I && ip <= ULAST_I)
				update_upperdisplay();
			else if(ip >= LFIRST_I && ip <= LLAST_I)
				update_lowerdisplay();
				
			touchwin(stdscr);
			break;

		case '+':
		case '-':
			switch(ip->type)
			{
				case ITEM_INT:
					if(c == '+')
						*((int *)ip->value) += 1;
					else if(*((int *)ip->value))
						*((int *)ip->value) -= 1;
					break;

				case ITEM_PSZ:
					if(c == '+')
						*((int *)ip->value) += 1;
					else if(*((int *)ip->value))
						*((int *)ip->value) -= 1;

					mpp += (ip->row - ROW_PA);
					changed_psz(mpp);
					break;					

				case ITEM_PBC:
					if(c == '+')
						*((int *)ip->value) += 1;
					else if(*((int *)ip->value))
						*((int *)ip->value) -= 1;

					mpp += (ip->row - ROW_PA);
					changed_pbc(mpp);
					break;					

				case ITEM_PEC:
					if(c == '+')
						*((int *)ip->value) += 1;
					else if(*((int *)ip->value))
						*((int *)ip->value) -= 1;

					mpp += (ip->row - ROW_PA);
					changed_pec(mpp);
					break;					

				case ITEM_PMB:
					if(c == '+')
						*((int *)ip->value) += 1;
					else if(*((int *)ip->value))
						*((int *)ip->value) -= 1;

					mpp += (ip->row - ROW_PA);
					changed_pmb(mpp);
					break;					
			}

			if(ip >= UFIRST_I && ip <= ULAST_I)
				update_upperdisplay();
			else if(ip >= LFIRST_I && ip <= LLAST_I)
				update_lowerdisplay();
				
			touchwin(stdscr);
			break;

		case ' ':		/* nextopt */
			if (ip->type == ITEM_ONOFF)
			{
				standout();
				if (*(ip->value))
				{
					*(ip->value) = 0;
					addstr(" NO");
				}
				else
				{
					*(ip->value) = 1;
					addstr("YES");
				}
				standend();
			}
			break;

		case CNTRL_D:
			ip = NULL;
			break;

		case KEY_UP:		/* up item */
			i = ip->col;	/* save current col */

			/* find destination row and put it in r */
			/* set ip to somewhere in destination row */
			
			if(ip->row == first_row)
			{
				r = last_row;
				do
					ip = ip->prev;
				while(ip->row != r);
			}
			else
			{
				r = ip->row;

				do
					ip = ip->prev;
				while(ip->row == r);

				r = ip->row;
			}

			/* goto start of destination row */
			
			while (ip->prev->row == r)
				ip = ip->prev;

			/* find column with smallest diff to previous column */

			ipsave = ip;
			diff = abs(i - ip->col);
 
			do
			{
				ip = ip->next;

				if(ip->type & ITEM_SPEC)
				{
					continue;
				}

				if((abs(i - ip->col)) < diff)
				{
					diff = abs(i - ip->col);
					ipsave = ip;
				}
			}
			while(ip->row == r);

			ip = ipsave;

			break;

		case KEY_DOWN:		/* down item */
			i = ip->col;	/* save current col */

			/* find destination row and put it in r */
			/* set ip to somewhere in destination row */
			
			if(ip->row == last_row)
			{
				r = first_row;
				do
					ip = ip->next;
				while(ip->row != r);
			}
			else
			{
				r = ip->row;

				do
					ip = ip->next;
				while(ip->row == r);

				r = ip->row;
			}

			/* goto start of destination row */
			
			while (ip->prev->row == r)
				ip = ip->prev;

			/* find column with smallest diff to previous column */

			ipsave = ip;
			diff = abs(i - ip->col);
 
			do
			{
				ip = ip->next;

				if(ip->type & ITEM_SPEC)
				{
					continue;
				}

				if((abs(i - ip->col)) < diff)
				{
					diff = abs(i - ip->col);
					ipsave = ip;
				}
			}
			while(ip->row == r);

			ip = ipsave;

			break;

		case KEY_LEFT:		/* left item */
#ifdef KEY_BTAB
		case KEY_BTAB:
#endif
			do
				ip = ip->prev;
			while(ip->type & ITEM_SPEC);
			break;

		case KEY_RIGHT:	/* right item */
		case TAB:
			do
				ip = ip->next;
			while(ip->type & ITEM_SPEC);
			break;

		case CNTRL_L:		/* refresh */
			touchwin(curscr);
			wrefresh(curscr);
			break;
	}
	return(ip);
}

/*---------------------------------------------------------------------------*
 *	user changed size, recalculate
 *---------------------------------------------------------------------------*/
static void
changed_psz(register menu_partition_t *mpp)
{

	if(dls.d_secpercyl)
	{
		mpp->p_beg = mpp->p_offset / dls.d_secpercyl;
		mpp->p_bega = mpp->p_offset % dls.d_secpercyl;	
		mpp->p_end = (mpp->p_offset + mpp->p_size + dls.d_secpercyl - 1) / dls.d_secpercyl - 1;
		mpp->p_enda = mpp->p_size % dls.d_secpercyl;
	}
	if((mpp->p_size * dls.d_secsize) >= 1048576)
		mpp->p_mb = (mpp->p_size * dls.d_secsize) / 1048576;
	else
		mpp->p_mb = 0;
}

/*---------------------------------------------------------------------------*
 *	user changed beginning cylinder, recalculate
 *---------------------------------------------------------------------------*/
static void
changed_pbc(register menu_partition_t *mpp)
{
	if(dls.d_secpercyl)
	{
		mpp->p_size = (mpp->p_end - mpp->p_beg) * dls.d_secpercyl;
		mpp->p_offset = mpp->p_beg * dls.d_secpercyl;
	/* 	mpp->p_beg = mpp->p_offset / dls.d_secpercyl; */
		mpp->p_bega = mpp->p_offset % dls.d_secpercyl;	
		mpp->p_end = (mpp->p_offset + mpp->p_size + dls.d_secpercyl - 1) / dls.d_secpercyl - 1;
		mpp->p_enda = mpp->p_size % dls.d_secpercyl;
	}
	if((mpp->p_size * dls.d_secsize) >= 1048576)
		mpp->p_mb = (mpp->p_size * dls.d_secsize) / 1048576;
	else
		mpp->p_mb = 0;
}

/*---------------------------------------------------------------------------*
 *	user changed ending cylinder, recalculate
 *---------------------------------------------------------------------------*/
static void
changed_pec(register menu_partition_t *mpp)
{
	if(dls.d_secpercyl)
	{
		mpp->p_size = (mpp->p_end - mpp->p_beg) * dls.d_secpercyl;
		mpp->p_offset = mpp->p_beg * dls.d_secpercyl;
		mpp->p_beg = mpp->p_offset / dls.d_secpercyl;
		mpp->p_bega = mpp->p_offset % dls.d_secpercyl;	
	/*	mpp->p_end = (mpp->p_offset + mpp->p_size + dls.d_secpercyl - 1) / dls.d_secpercyl - 1; */
		mpp->p_enda = mpp->p_size % dls.d_secpercyl;
	}
	if((mpp->p_size * dls.d_secsize) >= 1048576)
		mpp->p_mb = (mpp->p_size * dls.d_secsize) / 1048576;
	else
		mpp->p_mb = 0;
}

/*---------------------------------------------------------------------------*
 *	user changed size in megabyte, recalculate
 *---------------------------------------------------------------------------*/
static void
changed_pmb(register menu_partition_t *mpp)
{				
	if(dls.d_secpercyl)
	{
		mpp->p_size = (1048576 / dls.d_secsize) * mpp->p_mb;
		/* mpp->p_size = (mpp->p_end - mpp->p_beg) * dls.d_secpercyl; */
		mpp->p_offset = mpp->p_beg * dls.d_secpercyl;
		mpp->p_beg = mpp->p_offset / dls.d_secpercyl;
		mpp->p_bega = mpp->p_offset % dls.d_secpercyl;	
		mpp->p_end = (mpp->p_offset + mpp->p_size + dls.d_secpercyl - 1) / dls.d_secpercyl - 1;
		mpp->p_enda = mpp->p_size % dls.d_secpercyl;
	}
}

/*----------------------- E O F -------------------------------------------*/
