/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - items.c
 *	-------------------------------
 *
 *	last edit-date: [Sat Sep 21 14:53:07 1996]
 *
 *	-hm	stealing config routine from my good old cwish
 *
 *---------------------------------------------------------------------------*/

#include "defs.h"

/*---------------------------------------------------------------------------*
 *	initialization of the doubly linked list for all menu items
 *---------------------------------------------------------------------------*/

/* row, col, namestr, namelen, type, value, helpstr, nextitem, previtem */

struct item i_device = {
	ROW_0, 0, " device: ", 12, ITEM_DEVICE, (int *)(&device[0]),
	"currently selected device",
	&i_mount, &i_ph_mb
};

struct item i_mount = {
	ROW_0, 25, " state: ", 9, ITEM_STATE | ITEM_NOCHANGE, (int *)(&mountstate),
	"currently selected device",
	&i_command, &i_device
};

struct item i_command = {
	ROW_0, 45, "command: ", 12, ITEM_COMMAND, (int *)(&command),
	"what to do with the disklabel",
	&i_boot1, &i_mount
};

/* row, col, namestr, namelen, type, value, helpstr, nextitem, previtem */

struct item i_boot1 = {
	ROW_05, 0, "boot1: ", 30, ITEM_STRING, (int *)&dls.un_d_boot0, 
	"name of file containing the primary bootstrap program",
	&i_boot2, &i_command
};

struct item i_boot2 = {
	ROW_05, 40, "boot2: ", 30, ITEM_STRING, (int *)&dls.un_d_boot1, 
	"name of file containing the secondary bootstrap program",
	&i_type, &i_boot1
};

/* row, col, namestr, namelen, type, value, helpstr, nextitem, previtem */

struct item i_type = {
	ROW_1, 0, "   type: ", 7, ITEM_DRVTYPE, (int *)(&dls.d_type),
	"drive type, i.e. SCSI, ESDI, IDE etc.",
	&i_disk, &i_boot2
};

struct item i_disk = {
	ROW_1, 20, "   name: ", 16, ITEM_STRING, (int *)&dls.d_typename, 
	"disk type name (disktab entry name)",
	&i_label, &i_type
};

struct item i_label = {
	ROW_1, 46, "  label: ", 16, ITEM_STRING, (int *)&dls.d_mpackname, 
	"disk pack identifier label",
	&i_bps, &i_disk
};

/* row, col, namestr, namelen, type, value, helpstr, nextitem, previtem */

struct item i_bps = {
	ROW_2, 0, "byt/sec: ", 4, ITEM_BPS, (int *)&dls.d_secsize, 
	"number of bytes per sector",
	&i_spt, &i_label
};

struct item i_spt = {
	ROW_2, 20, "sec/trk: ", 4, ITEM_INT, (int *)&dls.d_nsectors, 
	"number of sectors per track",
	&i_tpc, &i_bps
};

struct item i_tpc = {
	ROW_2, 40, "trk/cyl: ", 4, ITEM_INT, (int *)&dls.d_ntracks, 
	"number of tracks per cylinder (equal to the number of data surfaces/heads)",
	&i_spc, &i_spt
};

struct item i_spc = {
	ROW_2, 60, "sec/cyl: ", 5, ITEM_INT, (int *)&dls.d_secpercyl, 
	"number of data sectors per cylinder",
	&i_cyl, &i_tpc
};

/* row, col, namestr, namelen, type, value, helpstr, nextitem, previtem */

struct item i_cyl = {
	ROW_3, 0, "cyl/dsk: ", 5, ITEM_INT, (int *)&dls.d_ncylinders, 
	"number of data cylinders per unit",
	&i_spu, &i_spc
};

struct item i_spu = {
	ROW_3, 20, "sec/dsk: ", 8, ITEM_INT, (int *)&dls.d_secperunit, 
	"number of data sectors per unit",
	&i_rpm, &i_cyl
};

struct item i_rpm = {
	ROW_3, 40, "    rpm: ", 5, ITEM_INT, (int *)&dls.d_rpm, 
	"rotational speed, rotations per minute",
	&i_ilv, &i_spu
};

struct item i_ilv = {
	ROW_3, 60, "interlv: ", 2, ITEM_INT, (u_int *)&dls.d_interleave, 
	"hardware sector interleave",
	&i_tks, &i_rpm
};

/* row, col, namestr, namelen, type, value, helpstr, nextitem, previtem */

struct item i_tks = {
	ROW_4, 0, "trkskew: ", 2, ITEM_INT, (u_int *)&dls.d_trackskew, 
	"sector 0 skew, per track",
	&i_cys, &i_ilv
};

struct item i_cys = {
	ROW_4, 20, "cylskew: ", 2, ITEM_INT, (u_int *)&dls.d_cylskew, 
	"sector 0 skew, per cylinder",
	&i_hsw, &i_tks
};

struct item i_hsw = {
	ROW_4, 40, " hdswtm: ", 8, ITEM_INT, (int *)&dls.d_headswitch, 
	"head switch time in microseconds",
	&i_tts, &i_cys
};

struct item i_tts = {
	ROW_4, 60, " seektm: ", 8, ITEM_INT, (int *)&dls.d_trkseek, 
	"track to track seek time in microseconds",
	&i_pa_sz, &i_hsw
};

/*======================================================================*/
/*	partition A							*/
/*======================================================================*/
struct item i_pa_sz = {
	ROW_PA, COL_SZ, "a: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_a.p_size, 
	"Partition a: size of partition in bytes",
	&i_pa_of, &i_tts
};

struct item i_pa_of = {
	ROW_PA, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_a.p_offset,
	"Partition a: offset (starting point) in bytes",
	&i_pa_ty, &i_pa_sz
};

struct item i_pa_ty = {
	ROW_PA, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_a.p_fstype,
	"Partition a: type of partition",
	&i_pa_fs, &i_pa_of
};

struct item i_pa_fs = {
	ROW_PA, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_a.p_fsize,
	"Partition a: fragment size in bytes",
	&i_pa_bs, &i_pa_ty
};

struct item i_pa_bs = {
	ROW_PA, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_a.p_bsize,
	"Partition a: block size in bytes",
	&i_pa_cpg, &i_pa_fs
};

struct item i_pa_cpg = {
	ROW_PA, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_a.p_mcpg,
	"Partition a: cylinder per group",
	&i_pa_beg, &i_pa_bs
};

struct item i_pa_beg = {
	ROW_PA, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_a.p_beg,
	"Partition a: first cylinder number",
	&i_pa_bega, &i_pa_cpg
};

struct item i_pa_bega = {
	ROW_PA, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_a.p_bega,
	"Partition a: first cylinder aligned flag (* = NOT aligned)",
	&i_pa_end, &i_pa_beg
};

struct item i_pa_end = {
	ROW_PA, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_a.p_end,
	"Partition a: last cylinder number",
	&i_pa_enda, &i_pa_bega
};

struct item i_pa_enda = {
	ROW_PA, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_a.p_enda,
	"Partition a: last cylinder aligned flag (* = NOT aligned)",
	&i_pa_mb, &i_pa_end
};

struct item i_pa_mb = {
	ROW_PA, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_a.p_mb,
	"Partition a: size of partition in megabytes",
	&i_pb_sz, &i_pa_enda
};

/*======================================================================*/
/*	partition B							*/
/*======================================================================*/
struct item i_pb_sz = {
	ROW_PB, COL_SZ, "b: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_b.p_size, 
	"Partition B, Size in Bytes",
	&i_pb_of, &i_pa_enda
};

struct item i_pb_of = {
	ROW_PB, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_b.p_offset,
	"Partition B, Offset in Bytes",
	&i_pb_ty, &i_pb_sz
};

struct item i_pb_ty = {
	ROW_PB, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_b.p_fstype,
	"Partition B, Type",
	&i_pb_fs, &i_pb_of
};

struct item i_pb_fs = {
	ROW_PB, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_b.p_fsize,
	"Partition B, Fragment Size in Bytes",
	&i_pb_bs, &i_pb_ty
};

struct item i_pb_bs = {
	ROW_PB, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_b.p_bsize,
	"Partition B, Fragments per Block",
	&i_pb_cpg, &i_pb_fs
};

struct item i_pb_cpg = {
	ROW_PB, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_b.p_mcpg,
	"Partition B, Cylinder per Group",
	&i_pb_beg, &i_pb_bs
};

struct item i_pb_beg = {
	ROW_PB, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_b.p_beg,
	"Partition B, Cylinder per Group",
	&i_pb_bega, &i_pb_cpg
};

struct item i_pb_bega = {
	ROW_PB, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_b.p_bega,
	"Partition B, Cylinder per Group",
	&i_pb_end, &i_pb_beg
};

struct item i_pb_end = {
	ROW_PB, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_b.p_end,
	"Partition B, Cylinder per Group",
	&i_pb_enda, &i_pb_bega
};

struct item i_pb_enda = {
	ROW_PB, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_b.p_enda,
	"Partition B, Cylinder per Group",
	&i_pb_mb, &i_pb_end
};

struct item i_pb_mb = {
	ROW_PB, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_b.p_mb,
	"Partition b: size of partition in megabytes",
	&i_pc_sz, &i_pb_enda
};

/*======================================================================*/
/*	partition C							*/
/*======================================================================*/
struct item i_pc_sz = {
	ROW_PC, COL_SZ, "c: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_c.p_size, 
	"Partition C, Size in Bytes",
	&i_pc_of, &i_pb_enda
};

struct item i_pc_of = {
	ROW_PC, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_c.p_offset,
	"Partition C, Offset in Bytes",
	&i_pc_ty, &i_pc_sz
};

struct item i_pc_ty = {
	ROW_PC, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_c.p_fstype,
	"Partition C, Type",
	&i_pc_fs, &i_pc_of
};

struct item i_pc_fs = {
	ROW_PC, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_c.p_fsize,
	"Partition C, Fragment Size in Bytes",
	&i_pc_bs, &i_pc_ty
};

struct item i_pc_bs = {
	ROW_PC, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_c.p_bsize,
	"Partition C, Fragments per Block",
	&i_pc_cpg, &i_pc_fs
};

struct item i_pc_cpg = {
	ROW_PC, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_c.p_mcpg,
	"Partition C, Cylinder per Group",
	&i_pc_beg, &i_pc_bs
};

struct item i_pc_beg = {
	ROW_PC, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_c.p_beg,
	"Partition C, Cylinder per Group",
	&i_pc_bega, &i_pc_cpg
};

struct item i_pc_bega = {
	ROW_PC, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_c.p_bega,
	"Partition C, Cylinder per Group",
	&i_pc_end, &i_pc_beg
};

struct item i_pc_end = {
	ROW_PC, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_c.p_end,
	"Partition C, Cylinder per Group",
	&i_pc_enda, &i_pc_bega
};

struct item i_pc_enda = {
	ROW_PC, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_c.p_enda,
	"Partition C, Cylinder per Group",
	&i_pc_mb, &i_pc_end
};

struct item i_pc_mb = {
	ROW_PC, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_c.p_mb,
	"Partition c: size of partition in megabytes",
	&i_pd_sz, &i_pc_enda
};

/*======================================================================*/
/*	partition D							*/
/*======================================================================*/
struct item i_pd_sz = {
	ROW_PD, COL_SZ, "d: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_d.p_size, 
	"Partition D, Size in Bytes",
	&i_pd_of, &i_pc_enda
};

struct item i_pd_of = {
	ROW_PD, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_d.p_offset,
	"Partition D, Offset in Bytes",
	&i_pd_ty, &i_pd_sz
};

struct item i_pd_ty = {
	ROW_PD, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_d.p_fstype,
	"Partition D, Type",
	&i_pd_fs, &i_pd_of
};

struct item i_pd_fs = {
	ROW_PD, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_d.p_fsize,
	"Partition D, Fragment Size in Bytes",
	&i_pd_bs, &i_pd_ty
};

struct item i_pd_bs = {
	ROW_PD, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_d.p_bsize,
	"Partition D, Fragments per Block",
	&i_pd_cpg, &i_pd_fs
};

struct item i_pd_cpg = {
	ROW_PD, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_d.p_mcpg,
	"Partition D, Cylinder per Group",
	&i_pd_beg, &i_pd_bs
};

struct item i_pd_beg = {
	ROW_PD, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_d.p_beg,
	"Partition D, Cylinder per Group",
	&i_pd_bega, &i_pd_cpg
};

struct item i_pd_bega = {
	ROW_PD, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_d.p_bega,
	"Partition D, Cylinder per Group",
	&i_pd_end, &i_pd_beg
};

struct item i_pd_end = {
	ROW_PD, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_d.p_end,
	"Partition D, Cylinder per Group",
	&i_pd_enda, &i_pd_bega
};

struct item i_pd_enda = {
	ROW_PD, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_d.p_enda,
	"Partition D, Cylinder per Group",
	&i_pd_mb, &i_pd_end
};

struct item i_pd_mb = {
	ROW_PD, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_d.p_mb,
	"Partition d: size of partition in megabytes",
	&i_pe_sz, &i_pd_enda
};

/*======================================================================*/
/*	partition E							*/
/*======================================================================*/
struct item i_pe_sz = {
	ROW_PE, COL_SZ, "e: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_e.p_size, 
	"Partition E, Size in Bytes",
	&i_pe_of, &i_pd_enda
};

struct item i_pe_of = {
	ROW_PE, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_e.p_offset,
	"Partition E, Offset in Bytes",
	&i_pe_ty, &i_pe_sz
};

struct item i_pe_ty = {
	ROW_PE, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_e.p_fstype,
	"Partition E, Type",
	&i_pe_fs, &i_pe_of
};

struct item i_pe_fs = {
	ROW_PE, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_e.p_fsize,
	"Partition E, Fragment Size in Bytes",
	&i_pe_bs, &i_pe_ty
};

struct item i_pe_bs = {
	ROW_PE, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_e.p_bsize,
	"Partition E, Fragments per Block",
	&i_pe_cpg, &i_pe_fs
};

struct item i_pe_cpg = {
	ROW_PE, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_e.p_mcpg,
	"Partition E, Cylinder per Group",
	&i_pe_beg, &i_pe_bs
};

struct item i_pe_beg = {
	ROW_PE, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_e.p_beg,
	"Partition E, Cylinder per Group",
	&i_pe_bega, &i_pe_cpg
};

struct item i_pe_bega = {
	ROW_PE, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_e.p_bega,
	"Partition E, Cylinder per Group",
	&i_pe_end, &i_pe_beg
};

struct item i_pe_end = {
	ROW_PE, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_e.p_end,
	"Partition E, Cylinder per Group",
	&i_pe_enda, &i_pe_bega
};

struct item i_pe_enda = {
	ROW_PE, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_e.p_enda,
	"Partition E, Cylinder per Group",
	&i_pe_mb, &i_pe_end
};

struct item i_pe_mb = {
	ROW_PE, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_e.p_mb,
	"Partition e: size of partition in megabytes",
	&i_pf_sz, &i_pe_enda
};

/*======================================================================*/
/*	partition F							*/
/*======================================================================*/
struct item i_pf_sz = {
	ROW_PF, COL_SZ, "f: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_f.p_size, 
	"Partition F, Size in Bytes",
	&i_pf_of, &i_pe_enda
};

struct item i_pf_of = {
	ROW_PF, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_f.p_offset,
	"Partition F, Offset in Bytes",
	&i_pf_ty, &i_pf_sz
};

struct item i_pf_ty = {
	ROW_PF, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_f.p_fstype,
	"Partition F, Type",
	&i_pf_fs, &i_pf_of
};

struct item i_pf_fs = {
	ROW_PF, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_f.p_fsize,
	"Partition F, Fragment Size in Bytes",
	&i_pf_bs, &i_pf_ty
};

struct item i_pf_bs = {
	ROW_PF, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_f.p_bsize,
	"Partition F, Fragments per Block",
	&i_pf_cpg, &i_pf_fs
};

struct item i_pf_cpg = {
	ROW_PF, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_f.p_mcpg,
	"Partition F, Cylinder per Group",
	&i_pf_beg, &i_pf_bs
};

struct item i_pf_beg = {
	ROW_PF, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_f.p_beg,
	"Partition F, Cylinder per Group",
	&i_pf_bega, &i_pf_cpg
};

struct item i_pf_bega = {
	ROW_PF, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_f.p_bega,
	"Partition F, Cylinder per Group",
	&i_pf_end, &i_pf_beg
};

struct item i_pf_end = {
	ROW_PF, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_f.p_end,
	"Partition F, Cylinder per Group",
	&i_pf_enda, &i_pf_bega
};

struct item i_pf_enda = {
	ROW_PF, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_f.p_enda,
	"Partition F, Cylinder per Group",
	&i_pf_mb, &i_pf_end
};

struct item i_pf_mb = {
	ROW_PF, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_f.p_mb,
	"Partition f: size of partition in megabytes",
	&i_pg_sz, &i_pf_enda
};

/*======================================================================*/
/*	partition G							*/
/*======================================================================*/
struct item i_pg_sz = {
	ROW_PG, COL_SZ, "g: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_g.p_size, 
	"Partition G, Size in Bytes",
	&i_pg_of, &i_pf_enda
};

struct item i_pg_of = {
	ROW_PG, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_g.p_offset,
	"Partition G, Offset in Bytes",
	&i_pg_ty, &i_pg_sz
};

struct item i_pg_ty = {
	ROW_PG, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_g.p_fstype,
	"Partition G, Type",
	&i_pg_fs, &i_pg_of
};

struct item i_pg_fs = {
	ROW_PG, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_g.p_fsize,
	"Partition G, Fragment Size in Bytes",
	&i_pg_bs, &i_pg_ty
};

struct item i_pg_bs = {
	ROW_PG, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_g.p_bsize,
	"Partition G, Fragments per Block",
	&i_pg_cpg, &i_pg_fs
};

struct item i_pg_cpg = {
	ROW_PG, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_g.p_mcpg,
	"Partition G, Cylinder per Group",
	&i_pg_beg, &i_pg_bs
};

struct item i_pg_beg = {
	ROW_PG, COL_BEG, "", LEN_BEG, ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_g.p_beg,
	"Partition G, Cylinder per Group",
	&i_pg_bega, &i_pg_cpg
};

struct item i_pg_bega = {
	ROW_PG, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_g.p_bega,
	"Partition G, Cylinder per Group",
	&i_pg_end, &i_pg_beg
};

struct item i_pg_end = {
	ROW_PG, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_g.p_end,
	"Partition G, Cylinder per Group",
	&i_pg_enda, &i_pg_bega
};

struct item i_pg_enda = {
	ROW_PG, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_g.p_enda,
	"Partition G, Cylinder per Group",
	&i_pg_mb, &i_pg_end
};

struct item i_pg_mb = {
	ROW_PG, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_g.p_mb,
	"Partition g: size of partition in megabytes",
	&i_ph_sz, &i_pg_enda
};

/*======================================================================*/
/*	partition H							*/
/*======================================================================*/
struct item i_ph_sz = {
	ROW_PH, COL_SZ, "h: ", LEN_SZ, ITEM_PSZ, (int *)&dls.part_h.p_size, 
	"Partition H, Size in Bytes",
	&i_ph_of, &i_pg_enda
};

struct item i_ph_of = {
	ROW_PH, COL_OF, "", LEN_OF, ITEM_INT | ITEM_NOCHANGE, (int *)&dls.part_h.p_offset,
	"Partition H, Offset in Bytes",
	&i_ph_ty, &i_ph_sz
};

struct item i_ph_ty = {
	ROW_PH, COL_TY, "", LEN_TY, ITEM_FSTYPE, (int *)&dls.part_h.p_fstype,
	"Partition H, Type",
	&i_ph_fs, &i_ph_of
};

struct item i_ph_fs = {
	ROW_PH, COL_FS, "", LEN_FS, ITEM_FSIZE, (int *)&dls.part_h.p_fsize,
	"Partition H, Fragment Size in Bytes",
	&i_ph_bs, &i_ph_ty
};

struct item i_ph_bs = {
	ROW_PH, COL_BS, "", LEN_BS, ITEM_BSIZE, (int *)&dls.part_h.p_bsize,
	"Partition H, Fragments per Block",
	&i_ph_cpg, &i_ph_fs
};

struct item i_ph_cpg = {
	ROW_PH, COL_CPG, "", LEN_CPG, ITEM_INT, (int *)&dls.part_h.p_mcpg,
	"Partition H, Cylinder per Group",
	&i_ph_beg, &i_ph_bs
};

struct item i_ph_beg = {
	ROW_PH, COL_BEG, "", LEN_BEG,ITEM_PBC | ITEM_NOCHANGE, (int *)&dls.part_h.p_beg,
	"Partition H, Cylinder per Group",
	&i_ph_bega, &i_ph_cpg
};

struct item i_ph_bega = {
	ROW_PH, COL_BEGA, "", LEN_BEGA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_h.p_bega,
	"Partition H, Cylinder per Group",
	&i_ph_end, &i_ph_beg
};

struct item i_ph_end = {
	ROW_PH, COL_END, "", LEN_END, ITEM_PEC, (int *)&dls.part_h.p_end,
	"Partition H, Cylinder per Group",
	&i_ph_enda, &i_ph_bega
};

struct item i_ph_enda = {
	ROW_PH, COL_ENDA, "", LEN_ENDA, ITEM_INDIC | ITEM_NOCHANGE, (int *)&dls.part_h.p_enda,
	"Partition H, Cylinder per Group",
	&i_ph_mb, &i_ph_end
};

struct item i_ph_mb = {
	ROW_PH, COL_MB, "", LEN_MB, ITEM_PMB, (int *)&dls.part_h.p_mb,
	"Partition h: size of partition in megabytes",
	FIRST_I, &i_ph_enda
};

/*---------------------------------- EOF ------------------------------------*/


