/*
 * Copyright (C) 1996 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	disk partition editor - main.c
 *	------------------------------
 *
 *	last edit-date: [Sat Sep 21 14:44:31 1996]
 *
 *	-hm	taking disklabel as an example
 *
 *---------------------------------------------------------------------------*/

#define MAIN
#include "defs.h"

static int first_row;
static int last_row;

static char *getehelp(int type);

int
main(int argc, char *argv[])
{
	struct utsname utsname;
	
	if((uname(&utsname)) == -1)
	{
		perror("dpe: uname failed");
		exit(1);
	}

	if(strcmp(utsname.machine, "i386"))
	{
		fprintf(stderr, "dpe: dpe runs ONLY on i386 platforms!\n");
		exit(1);
	}

	if(getuid() != 0)
	{
		fprintf(stderr, "dpe: you must be superuser to run dpe!\n");
		exit(1);
	}
		
	if(get_mounted() == 0)
		exit(98);
		
	display();

	return(0);
}

/*---------------------------------------------------------------------------*
 *	run-time configuration menu
 *---------------------------------------------------------------------------*/
void
display(void)
{
	struct item *ip;
	struct item *oldip;	
	int c;
	int i;

	label_to_int(&dls, lp);	/* convert disklabel to internal struct */

	/* init curses */
	
	initscr();
	nonl();                                 /* optimize */
	raw();                                  /* raw input */
	noecho();                               /* do not echo input */
	idlok(stdscr, TRUE);                    /* use insert/delete */
	keypad(stdscr, TRUE);                   /* use special keys */

	if(COLS < 80 || LINES < 24)
	{
		endwin();
		fprintf(stderr, "dpe: display must at least be 80x24!\n");
		exit(1);
	}
	
	/* print header */
	                                                
	hdrline(0);

	/* set item pointer */
	
	ip = FIRST_I;

	first_row = ip->row;
	
	/* display menu */

	do
	{
		display_item(ip, 0);

		if(ip->next == FIRST_I)
			last_row = ip->row;
	}
	while ((ip = ip->next) != FIRST_I);

	/* print separator lines */
	
	separate(LINE_SEP0);
	separate(LINE_SEP1);
	separate(LINE_SEP2);
	
	/* print partition header line */
	
	move(LINE_PARTHD, 0);
	addstr(HEADER_PART);
	
	/* reset item pointer */
	
	ip = FIRST_I;

	/* cruise in menu */

	i = 0;

	oldip = ip;
	
	for (;;)
	{
		char *ehelp;
		
		/* clear standout on old item */
		
		display_item(oldip, 0);

		oldip = ip;		

		/* display item help line */
		
		move(LINE_IHELP, 0);
		clrtoeol();
		move(LINE_IHELP, (COLS - strlen(ip->help)) / 2);
		addstr(ip->help);

		/* display edit help */
		
		move(LINE_IHELP+1, 0);
		clrtoeol();
		ehelp = getehelp(ip->type);
		move(LINE_IHELP+1, (COLS - strlen(ehelp)) / 2);
		addstr(ehelp);

		/* display current item standout */
		
		display_item(ip, 1);

		move(0, COLS-1);

		refresh();

		c = getch();

		if((ip = edit_item(ip, c)) == NULL)
		{
			break;
		}
	}

	wmove(stdscr, 0, 0);			/* first line */
	wclear(stdscr);				/* clear window */
	touchwin(stdscr);
	wnoutrefresh(stdscr);

	endwin();
}

/*---------------------------------------------------------------------------*
 *	execute a command from the command menu
 *---------------------------------------------------------------------------*/
void execute(int cmd)
{
	static char filename[256] = "/var/tmp/disklabel.dpe";
	static char disktabentry[80] = "";
	struct disklabel rwdl;
	FILE *fp;
	int f;
	
	touchwin(stdscr);
	refresh();

	bzero(&rwdl, sizeof(struct disklabel));
	
	switch(cmd)
	{
		case CMD_EXIT:
			endwin();
			exit(0);
			break;
			
		case CMD_RD_DLF:	/* read disklabel file */
			curgets(filename, COLS-5, LINES/2, 2,
				"read disklabel filename", 0, filename);
			fp = fopen(filename, "r");
			getasciilabel(fp, &rwdl);
			fclose(fp);
			label_to_int(&dls, &rwdl);
			update_display();
			break;

		case CMD_WR_DLF:	/* write disklabel file */
			curgets(filename , COLS-5, LINES/2, 2,
				"write disklabel filename", 0, filename);
			int_to_label(&dls, &rwdl);
			fp = fopen(filename, "w");
			putasciilabel(fp, &rwdl);
			fclose(fp);
			break;

		case CMD_RD_DTB:
			curgets(disktabentry, 20, LINES/2, (COLS/2)-10,
				"get disktab entry", 0, disktabentry);
			getdisktab(disktabentry, NULL, &rwdl);
			label_to_int(&dls, &rwdl);
			update_display();
			break;
			
		case CMD_AUTO:
			getdisktab("auto", NULL, &rwdl);
			label_to_int(&dls, &rwdl);
			update_display();
			break;
			
		case CMD_WR_LB:
			installboot = 0;
			if(device[0] == '\0')
			{
				Error("device not specified!");
				break;
			}
			if(mountstate)
			{
				Error("device is currrently mounted and in use!");
				break;
			}

			int_to_label(&dls, &rwdl);			

			if((f = open(specname, O_RDWR)) < 0)
			{
				Perror(specname);
				return;
			}

			lp = makebootarea(bootarea, &rwdl, f);

			*lp = rwdl;
			if (checklabel(lp) == 0)
				writelabel(f, bootarea, lp);
			close(f);
			break;

		case CMD_WR_ALL:
			installboot = 1;
			if(device[0] == '\0')
			{
				Error("device not specified!");
				break;
			}
			if(mountstate)
			{
				Error("device is currrently mounted and in use!");
				break;
			}

			int_to_label(&dls, &rwdl);			

			if((f = open(specname, O_RDWR)) < 0)
			{
				Perror(specname);
				return;
			}

			lp = makebootarea(bootarea, &rwdl, f);

			*lp = rwdl;
			if (checklabel(lp) == 0)
				writelabel(f, bootarea, lp);
			close(f);
			break;
	}
}

#define MAXHELPSTR 256
static char *getehelp(int type)
{
	static char string[MAXHELPSTR];
	
	switch(type & ITEM_MASK)
	{
		case ITEM_ONOFF:	/* YES / NO */
			strcpy(string, "SPACE key toggles between YES and NO");
			break;
		case ITEM_STRING:	/* a string */
			strcpy(string, "<CR> opens a box to enter a string");
			break;
		case ITEM_CHAR:		/* a character */
			strcpy(string, "enter a character");
			break;
		case ITEM_INT:		/* an integer */
			strcpy(string, "<CR> openes a box to enter a number. '+' increments, '-' decrements");
			break;
		case ITEM_INDIC:	/* an indicator */
			strcpy(string, "just an indicator");
			break;
		case ITEM_FSTYPE:	/* FS type */
			strcpy(string, "<CR> displays a menu to choose a filesystem type");
			break;
		case ITEM_DRVTYPE:	/* disk drive type */
			strcpy(string, "<CR> displays a menu to choose a drive type");
			break;
		case ITEM_DEVICE:	/* which device we're on */
			strcpy(string, "<CR> openes a box to enter the device");
			break;
		case ITEM_COMMAND:	/* what to do */
			strcpy(string, "<CR> openes the command menu");
			break;
		case ITEM_STATE:	/* mounted ? */
			strcpy(string, "displays the drive status");
			break;
		case ITEM_BPS:		/* byte per sect */
			strcpy(string, "<CR> openes a menu with valid sector values.");
			break;
		case ITEM_FSIZE:	/* frag size */
			strcpy(string, "<CR> displays a menu to choose a fragment size. '+' increments, '-' decrements");
			break;
		case ITEM_BSIZE:	/* block size */
			strcpy(string, "<CR> displays a menu to choose a block size. '+' increments, '-' decrements");
			break;
		case ITEM_PSZ:		/* partition size */
			strcpy(string, "<CR> openes a box to enter a number. '+' increments, '-' decrements");
			break;
		case ITEM_PBC:		/* partition beg cyl */
			strcpy(string, "<CR> openes a box to enter a number. '+' increments, '-' decrements");
			break;
		case ITEM_PEC:		/* partition end cyl */
			strcpy(string, "<CR> openes a box to enter a number. '+' increments, '-' decrements");
			break;
		case ITEM_PMB:		/* partition mbyte */ 
			strcpy(string, "<CR> openes a box to enter a number. '+' increments, '-' decrements");
			break;

	}
	return(&string[0]);
}

/*----------------------- E O F -------------------------------------------*/
